/*	GetBitmap.cpp		*/

#include "GetBitmap.h"
#include "Datatypes.h"
#include "DataStreams.h"
#include "BitmapStream.h"


/*
 *	This function may come in handy one day
 *
static bool
Retry(
	long				error)
{
	static bool didInit = false;

	if (error == B_INTERRUPTED)
		return TRUE;
	if ((error == DATA_NOT_INITIALIZED) && !didInit) {
		didInit = true;
		if (!DATAInit('????'))
			return true;
	}
	return false;
}
*/

BBitmap *
GetBitmap(
	const char			*name)
{
	record_ref ref;
	BBitmap *ret = NULL;
	if (!get_ref_for_path(name, &ref))
		ret = GetBitmap(ref);
	if (!ret) {
		ret = GetBitmap(DATA_BITMAP, name);
	}
	return ret;
}


BBitmap *
GetBitmap(
	record_ref			ref)
{
	BFile file;
	if (file.SetRef(ref))
		return NULL;
	ulong type = 0, app = 0;
	if (file.GetTypeAndApp(&type, &app))
		return NULL;
	DFileStream input(&file);
	input.SetDispose(false);
	BitmapStream output;
	if (DATATranslate(input, NULL, NULL, output, DATA_BITMAP, type))
		if (type && DATATranslate(input, NULL, NULL, output, DATA_BITMAP))
			return NULL;
	BBitmap *ret = NULL;
	if (output.DetachBitmap(ret))
		return NULL;
	return ret;
}


BBitmap *
GetBitmap(
	ulong				type,
	long				id)
{
	if (!be_app)
		return NULL;
	app_info info;
	be_app->GetAppInfo(&info);
	BResourceFile file(info.ref);
	if (file.Open(B_READ_ONLY))
		return NULL;
	long size = 0;
	void *data = file.FindResource(type, id, &size);
	file.Close();
	if (!data)
		return NULL;
	DMallocStream input(data, size);
	BitmapStream output;
	if (DATATranslate(input, NULL, NULL, output, DATA_BITMAP))
		return NULL;
	BBitmap *ret = NULL;
	if (output.DetachBitmap(ret))
		return NULL;
	return ret;
}


BBitmap *
GetBitmap(
	ulong				type,
	const char			*name)
{
	if (!be_app)
		return NULL;
	app_info info;
	be_app->GetAppInfo(&info);
	BResourceFile file(info.ref);
	if (file.Open(B_READ_ONLY))
		return NULL;
	long size = 0;
	void *data = file.FindResource(type, name, &size);
	file.Close();
	if (!data)
		return NULL;
	DMallocStream input(data, size);
	BitmapStream output;
	if (DATATranslate(input, NULL, NULL, output, DATA_BITMAP))
		return NULL;
	BBitmap *ret = NULL;
	if (output.DetachBitmap(ret))
		return NULL;
	return ret;
}
