/*	AppMakeView.cpp
 */

#include "Datatypes.h"
#include "AppMakeView.h"
#include <TextView.h>
#include "DataStreams.h"
#include "BitmapStream.h"
#include "BitmapView.h"
#include "PictureView.h"
#include "MediaPlayerView.h"



long
AppMakeView(
	DStream *		inStream,
	BView * &		outView,
	BRect &			outExtent,
	unsigned long	hintType,
	char * &		outDataInfo)	//	call delete[]
{
	outDataInfo = NULL;
	outView = NULL;
	long err = B_ERROR;
	DATAInfo info;
	/*	First try to identify the data using the hint, if any
	 */
	if (hintType) {
		err = DATAIdentify(*inStream, NULL, info, hintType);
	}
	/*	If not identified, try without a hint
	 */
	if (err) {
		err = DATAIdentify(*inStream, NULL, info);
	}
	/*	Bail out if we can't find anything
	 */
	if (err)
		return err;

	outDataInfo = new char[strlen(info.formatName)+1];
	strcpy(outDataInfo, info.formatName);

	/*	Call through to helper functions for the type found
	 */
	switch (info.formatGroup) {
	case DATA_TEXT:
		err = AppMakeTextView(outView, outExtent, *inStream, info);
		if (!err && outView)
			delete inStream;
		break;
	case DATA_BITMAP:
		err = AppMakeBitmapView(outView, outExtent, *inStream, info);
		if (!err && outView)
			delete inStream;
		break;
	case DATA_PICTURE:
		err = AppMakePictureView(outView, outExtent, *inStream, info);
		if (!err && outView)
			delete inStream;
		break;
	case DATA_MEDIA:
		err = AppMakeMediaView(outView, outExtent, *inStream, info);
		break;	/*	saves stream for later use	*/
	default:
		err = B_ERROR;	/*	A format that's not text, bitmap or picture */
		break;
	}
	return err;
}


long
AppMakeTextView(
	BView * &		outView,
	BRect &			outExtent,
	DStream &		stream,
	DATAInfo &		info)
{
	/*	Translate */
	DMallocStream mall;
	long err = DATATranslate(stream, &info, NULL, mall, DATA_TEXT);
	if (err)
		return err;

	/*	Create view */
	outExtent.left = outExtent.top = 0;
	outExtent.right = 400;
	outExtent.bottom = 300;
	outView = new BTextView(outExtent, "text", outExtent,
			B_FOLLOW_ALL, B_WILL_DRAW | B_PULSE_NEEDED);

	/*	Insert data */
	long size = 0;
	long offset = 0;
	const char *text = (const char *)mall.Peek(offset, size);
	((BTextView *)outView)->SetText(text+offset, size);

	/*	Declare success */
	return B_NO_ERROR;
}


long
AppMakeBitmapView(
	BView * &		outView,
	BRect &			outExtent,
	DStream &		stream,
	DATAInfo &		info)
{
	BitmapStream bms;
	long err = DATATranslate(stream, &info, NULL, bms, DATA_BITMAP);
	if (err)
		return err;

	/*	Get data */
	BBitmap *map = NULL;
	err = bms.DetachBitmap(map);
	if (err)
		return err;

	/*	Create view */
	outExtent = map->Bounds();
	BitmapView *view = new BitmapView(map);
	outView = view;

	/*	Declare success */
	return B_NO_ERROR;
}


long
AppMakePictureView(
	BView * &		outView,
	BRect &			outExtent,
	DStream &		stream,
	DATAInfo &		info)
{
	DMallocStream mall;
	long err = DATATranslate(stream, &info, NULL, mall, DATA_PICTURE);
	if (err)
		return err;

	/*	Get data */
	long offset = 0;
	long size = 0;
	const void *data = mall.Peek(offset, size);
	BPicture *pic = new BPicture((char *)data+offset, size);

	/*	Create view */
	/*	outExtent = pic->Bounds();	//	Can't get bounds of picture!
	 *	so we just fake some
	 */
	outExtent.left = outExtent.top = 0;
	outExtent.right = 512;
	outExtent.bottom = 384;
	PictureView *view = new PictureView(outExtent, pic);
	outView = view;

	/*	Declare success */
	return B_NO_ERROR;
}



long
AppMakeMediaView(
	BView * &		outView,
	BRect &			outExtent,
	DStream &		stream,
	DATAInfo &		info)
{
	outExtent.Set(0,0, 320,240);
	MediaPlayerView *view = new MediaPlayerView(outExtent, &stream, info);
	outView = view;
	return B_OK;
}
