/*	BitmapView.cpp
 */

#include "BitmapView.h"
#include <Bitmap.h>
#include <ScrollBar.h>


BitmapView::BitmapView(
	BBitmap *		map) :
	BView(map->Bounds(), "bitmap", B_FOLLOW_ALL, B_WILL_DRAW | B_FRAME_EVENTS),
	fMargins(-20,-20,20,20)
{
	fBitmap = map;
	fOwnership = true;
	fAdjustBars = true;
	rgb_color gray = { 220, 220, 220, 0 };
	SetViewColor(gray);
}


BitmapView::~BitmapView()
{
	if (fOwnership && fBitmap)
		delete fBitmap;
}


void
BitmapView::SetOwnership(
	bool			ownership)
{
	fOwnership = ownership;
}


bool
BitmapView::GetOwnership()
{
	return fOwnership;
}


void
BitmapView::SetBitmap(
	BBitmap *		map)
{
	if (fOwnership && fBitmap)
		delete fBitmap;
	fBitmap = map;
	Invalidate();
	if (fAdjustBars)
		AdjustBars();
}


BBitmap *
BitmapView::GetBitmap()
{
	return fBitmap;
}


void
BitmapView::SetMargins(
	BRect			margins)
{
	fMargins = margins;
	if (fAdjustBars)
		AdjustBars();
}


BRect
BitmapView::GetMargins()
{
	return fMargins;
}


void
BitmapView::SetAdjustBars(
	bool			adjust)
{
	fAdjustBars = adjust;
	if (fAdjustBars)
		AdjustBars();
}


bool
BitmapView::GetAdjustBars()
{
	return fAdjustBars;
}


void
BitmapView::Draw(
	BRect			area)
{
	if (!fBitmap)
		return;
	BRect frame = fBitmap->Bounds();
	area = area & frame;
	if ((area.right >= area.left) && (area.bottom >= area.top))
		DrawBitmap(fBitmap, area, area);
	frame.InsetBy(-1, -1);
	StrokeRect(frame);
}


void
BitmapView::FrameResized(
	float			newWidth,
	float			newHeight)
{
	BView::FrameResized(newWidth, newHeight);
	if (fAdjustBars)
		AdjustBars();
}


	static void
	AdjustBar(
		BScrollBar *		bar,
		float				page,
		float				total,
		float				start)
	{
		if (total <= page) {
			bar->SetRange(start, start);
		} else {
			bar->SetRange(start, start+total-page);
		}
		float pgStep = page-4.0;
		if (pgStep<16.0)
			pgStep = 16.0;
		bar->SetSteps(4.0, pgStep);
	}

void
BitmapView::AdjustBars()
{
	BRect area(0,0,0,0);
	BRect bounds(Bounds());
	if (fBitmap)
		area = fBitmap->Bounds();

	BScrollBar *bar = ScrollBar(B_HORIZONTAL);
	if (bar)
		AdjustBar(bar, bounds.Width(), area.Width()+fMargins.Width()+1, fMargins.left);
	bar = ScrollBar(B_VERTICAL);
	if (bar)
		AdjustBar(bar, bounds.Height(), area.Height()+fMargins.Height()+1, fMargins.top);
}


void
BitmapView::WindowActivated(
	bool			state)
{
	BView::WindowActivated(state);
	if (fAdjustBars && state) {
		AdjustBars();
	}
}






