//	ShowApp.cpp

#include "ShowApp.h"
#include "Datatypes.h"
#include "DataStreams.h"
#include "AppMakeView.h"

#include <Window.h>
#include <ScrollBar.h>
#include <File.h>


#ifndef max
#define max(a,b) ((a)>(b)?(a):(b))
#endif

ShowApp::ShowApp(
	ulong sig) :
	BApplication(sig)
{
	SetPulseRate(500000);
	if (DATAVersion == NULL) {
		BAlert *alrt = new BAlert("", "Datatypes.lib is not installed!", "Quit");
		alrt->Go();
		PostMessage(B_QUIT_REQUESTED);
		return;
	}
	long curv, minv;
	DATAVersion(curv, minv);
	if (curv < DATA_MIN_VERSION) {
		BAlert *alrt = new BAlert("", "Datatypes.lib is too old!", "Quit");
		alrt->Go();
		PostMessage(B_QUIT_REQUESTED);
		return;
	}
	if (minv > DATA_CURRENT_VERSION) {
		BAlert *alrt = new BAlert("", "Datatypes.lib is too new!?", "Quit");
		alrt->Go();
		PostMessage(B_QUIT_REQUESTED);
		return;
	}
	long err = DATAInit('SHOW');
	if (err) {
		char message[100];
		sprintf(message, "Datatypes.lib got error 0x%08x when initializing!", "Quit");
		BAlert *alrt = new BAlert("", message, "Quit");
		alrt->Go();
		PostMessage(B_QUIT_REQUESTED);
		return;
	}
}


void
ShowApp::RefsReceived(
	BMessage *message)
{
	screen_info info;
	get_screen_info(&info);
	BRect winR = info.frame;
	winR.left += 64;
	get_dock_width(&winR.left);
	winR.left += 6;
	winR.right -= 4;
	winR.top += 26;
	winR.bottom -= 4;

	for (int ix=0; message->HasRef("refs", ix); ix++) {

		/*	get the file out of the message */

		record_ref ref = message->FindRef("refs", ix);
		if (!does_ref_conform(ref, "File")) {
			BAlert *error = new BAlert("", "This is not a file!", "OK");
			error->Go();
			continue;
		}

		BFile *file = new BFile(ref);
		ulong type, app;
		if (file->GetTypeAndApp(&type, &app)) {
			BAlert *error = new BAlert("", "Error finding file!", "OK");
			error->Go();
			continue;
		}

		/*	try to create a view using type hint */

		char name[B_FILE_NAME_LENGTH];
		file->GetName(name);

		BView *outView = NULL;
		char *dataInfo;
		BRect extent;
		DFileStream * stream = new DFileStream(file, B_READ_ONLY);
		long err = AppMakeView(stream, outView, extent, type, dataInfo);

		/*	if that doesn't work, use (slower) content identification */

		if (err || !outView) {
			BAlert *error = new BAlert("", "No data type handler found!", "OK");
			error->Go();
			delete stream;
			continue;
		}

		/* Put this view in a window with scroll bars */

		BRect winSize = winR;
		if (winSize.Width() > extent.Width()+B_V_SCROLL_BAR_WIDTH)
			winSize.right = winSize.left+extent.Width()+B_V_SCROLL_BAR_WIDTH;
		if (winSize.Height() > extent.Height()+B_H_SCROLL_BAR_HEIGHT)
			winSize.bottom = winSize.top+extent.Height()+B_H_SCROLL_BAR_HEIGHT;
		BWindow *window = new BWindow(winSize, name, B_DOCUMENT_WINDOW, 0);
		window->Lock();
		BRect viewR = window->Bounds();
		viewR.right -= B_V_SCROLL_BAR_WIDTH;
		viewR.bottom -= B_H_SCROLL_BAR_HEIGHT;

		outView->MoveTo(0,0);
		outView->ResizeTo(viewR.right, viewR.bottom);
		window->AddChild(outView);

		BRect tR = viewR;
		tR.top = tR.bottom+1;
		tR.bottom += 15;
		tR.right++;
		tR.left += 120;
		BScrollBar *hBar = new BScrollBar(tR, "horiz", outView,
			0, max(extent.Width()-viewR.Height(), 0), B_HORIZONTAL);
		hBar->SetSteps(8, 240);
		window->AddChild(hBar);

		tR.right = tR.left;
		tR.left = -1;
		BBox *box = new BBox(tR, NULL, B_FOLLOW_LEFT|B_FOLLOW_BOTTOM);
		window->AddChild(box);
		rgb_color dkGray = { 150, 150, 150, 0 };
		box->SetHighColor(dkGray);
		tR.InsetBy(1, 1);
		tR.OffsetTo(BPoint(1,1));
		BStringView *str = new BStringView(tR, "info", dataInfo, B_FOLLOW_ALL);
		box->AddChild(str);
		rgb_color gray = { 220, 220, 220, 0 };
		str->SetViewColor(gray);
		str->SetDrawingMode(B_OP_OVER);
		str->SetFontName("Erich");
		str->SetFontSize(9);
		delete[] dataInfo;

		tR = viewR;
		tR.left = tR.right+1;
		tR.right += 15;
		tR.bottom++;
		tR.top--;
		BScrollBar *vBar = new BScrollBar(tR, "vert", outView,
			0, max(extent.Height()-viewR.Height(), 0), B_VERTICAL);
		vBar->SetSteps(8, 160);
		window->AddChild(vBar);

		window->Unlock();
		window->Show();
		winR.OffsetBy(5, 15);

		/*	Since the user may resize the window, we should put 
			the view inside another view that adjusts the scroll 
			bars, but I'm too lazy for that right now. */
	}
}


/*
   I would do the check in AppActivated, but there is a 
   race condition in where the window list still contains 
   a window when the application gets deactivated
 */
void
ShowApp::Pulse()
{
	BApplication::Pulse();
	if (!CountWindows())
		PostMessage(B_QUIT_REQUESTED);
}


void
ShowApp::ArgvReceived(
	int argc,
	char **argv)
{
	BMessage *msg = new BMessage(B_REFS_RECEIVED);
	for (int ix=1; ix<argc; ix++) {
		record_ref ref;
		if (!get_ref_for_path(argv[ix], &ref)) {
			msg->AddRef("refs", ref);
		}
	}
	if (msg->HasRef("refs"))
		PostMessage(msg);
	else
		delete msg;
}
